/**
 * @file 	pac_afemux.c
 * @version v1.0.0
 * @date	14 October 2022
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"
#include "pac_ADC.h"
#include "pac_driver_adc.h"
#include "pac_afemux.h"

//===============================================================================================================================
// The PAC25140 contains the AFE MUX that can be used to sample the internal power supply rails on the device, internal die
// temperature, and other signals.
// This app shows the basic configure of AFE MUX 
//===============================================================================================================================

void pac_afemux_monitor_init(void)
{
	// Configure PC0 as ADC input
	pac_adc_config_io(0x01);
	//Configure ADC mode:single  repeat:no
	//FCLK=50Mhz, ADCCLK=50Mhz/4=12.5Mhz
		pac_adc_config(ADCCTL_MODE_MANUAL, ADCCTL_CLKDIV_DIV4, 0);
    //pac_adc_set_mux(ADCCTL_ADMUX_AD0);
		PAC_ADC->ADCCTL.CHANNEL = ADCCTL_ADMUX_AD0;
	
    pac_adc_enable(1);

	pac_tile_socbridge_config(1, 0);

    afe_set_mcu_alive(1);
    afe_signal_manager_enable(1);
    afe_set_driver_mode(DRVCTL_DRVMODE_ENABLED);
    afe_charge_pump_enable(1);

    // Configure AFEMUX
    afe_afemux_config(0, 1);
    afe_afemux_select_mux(MONITOR_AFEMUX);
}

void pac_afemux_monitor(void)
{
    pac_adc_start();
    while (PAC_ADC->ADCCTL.ADBUSY);

    switch (MONITOR_AFEMUX)
    {
    case AFEMUXSEL_AFEMUX_VCORE:
        printf("VCORE: ");
        break;
    case AFEMUXSEL_AFEMUX_VCORE_DIV2P5:
        printf("VCORE x 4/10: ");
        break;
    case AFEMUXSEL_AFEMUX_VDDA_DIV2P5:
        printf("VDDA x 4/10: ");
        break;
    case AFEMUXSEL_AFEMUX_VCCIO_DIV2P5:
        printf("VCCIO x 4/10: ");
        break;
    case AFEMUXSEL_AFEMUX_VSYS_DIV2P5:
        printf("VSYS x 4/10: ");
        break;
    case AFEMUXSEL_AFEMUX_ISENSE:
        printf("ISENSE: ");
        break;
    case AFEMUXSEL_AFEMUX_VPTAT:
        printf("VPTAT: ");
        break;
    case AFEMUXSEL_AFEMUX_VP_DIV10:
        printf("VP x 1/10: ");
        break;
    case AFEMUXSEL_AFEMUX_VREF_DIV2:
        printf("VREF x 1/2: ");
        break;
    case AFEMUXSEL_AFEMUX_FUSE_DIV10:
        printf("FUSE x 1/10: ");
        break;
    case AFEMUXSEL_AFEMUX_CHG_DIV50:
        printf("CHG x 1/50: ");
        break;
    case AFEMUXSEL_AFEMUX_DSG_DIV50:
        printf("DSG x 1/50: ");
        break;
    case AFEMUXSEL_AFEMUX_BAT_DIV50:
        printf("BAT x 1/50: ");
        break;
    case AFEMUXSEL_AFEMUX_AIO0A:
        printf("AIOA: ");
        break;
    case AFEMUXSEL_AFEMUX_LOADDET:
        printf("LOADDET: ");
        break;
    case AFEMUXSEL_AFEMUX_SCPDAC:
        printf("SCPDAC: ");
        break;
    case AFEMUXSEL_AFEMUX_OCCDAC:
        printf("OCCDAC: ");
        break;
    case AFEMUXSEL_AFEMUX_OCDDAC:
        printf("OCDDAC: ");
        break;
    case AFEMUXSEL_AFEMUX_BATOVDAC_DIV2:
        printf("BATOVDAC x 1/2: ");
        break;
    case AFEMUXSEL_AFEMUX_VIN_DIV50:
        printf("VIN x 1/50: ");
        break;
    case AFEMUXSEL_AFEMUX_PACK_DIV50:
        printf("PACK+ x 1/50: ");
        break;
    case AFEMUXSEL_AFEMUX_VCP_DIV50:
        printf("VCP x 1/50: ");
        break;
    default:
        printf("Invalid AFEMUX!!!");
        return;
    }

    printf("%d \n",PAC_ADC->ADCRES.VAL);
}


