/**
 * @file 	pac_hibernate.c
 * @version v1.0.0
 * @date	14 October 2022
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"
#include "pac_ADC.h"
#include "pac_driver_adc.h"
#include "pac_hibernate.h"

void pac_hibernate_init(void)
{
	// Configure SOC Bridge for talking to AFE
	pac_tile_socbridge_config(1, 0);

    // Configure AFE
    afe_set_mcu_alive(1);
    afe_signal_manager_enable(1);
    afe_set_driver_mode(DRVCTL_DRVMODE_ENABLED);
    afe_charge_pump_enable(1);

    // Configure Hibernate
#if (WAKEUP_SOURCE == WAKEUP_PB)
    afe_pushbtn_config(0, 1, PB_DEGLITCH_TIME_8MS, PB_POLARITY_ACTIVE_LOW, 0);
    afe_pushbtn_enable(1);
    afe_hibernate_config(1, 0, HIBCTL_PACKWAKEVREF_5V, HIBCTL_WAKESRC_PB, HIBCTL_WAKETIMER_DISABLED);
#elif (WAKEUP_SOURCE == WAKEUP_PACK)
    afe_hibernate_config(0, 1, HIBCTL_PACKWAKEVREF_5V, HIBCTL_WAKESRC_PACK, HIBCTL_WAKETIMER_DISABLED);
#elif (WAKEUP_SOURCE == WAKEUP_WUTIMER)
    afe_hibernate_config(0, 0, HIBCTL_PACKWAKEVREF_5V, HIBCTL_WAKESRC_WUTIMER, HIBCTL_WAKETIMER_4000MS);
#else   // WAKEUP_SOURCE == RFU
    afe_hibernate_config(0, 0, HIBCTL_PACKWAKEVREF_5V, HIBCTL_WAKESRC_RFU, HIBCTL_WAKETIMER_DISABLED);
#endif

}


