/**
 * @file 	pac_aio0.c
 * @version v1.0.0
 * @date	14 October 2022
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"
#include "pac_ADC.h"
#include "pac_driver_adc.h"
#include "pac_aio0.h"

void pac_aio0_init(void)
{
	// Configure SOC Bridge for talking to AFE
	pac_tile_socbridge_config(1, 0);

    // Configure AFE
    afe_set_mcu_alive(1);
    afe_signal_manager_enable(1);
    afe_set_driver_mode(DRVCTL_DRVMODE_ENABLED);
    afe_charge_pump_enable(1);

    // Configure AIO0
#if (AIO0_MODE == AIO0_MODE_INPUT)
	// Configure PC0 as ADC input
	pac_adc_config_io(0x01);
	//Configure ADC mode:single  repeat:no
	//FCLK=50Mhz, ADCCLK=50Mhz/4=12.5Mhz
	pac_adc_config(ADCCTL_MODE_MANUAL, ADCCTL_CLKDIV_DIV4, 0);
	//pac_adc_set_mux(ADCCTL_ADMUX_AD0);
	PAC_ADC->ADCCTL.CHANNEL = ADCCTL_ADMUX_AD0;
    pac_adc_enable(1);

    // Configure AFEMUX = AIO0
    afe_afemux_config(0, 1);
    afe_afemux_select_mux(AFEMUXSEL_AFEMUX_AIO0A);

    // Configure AIO
    afe_aio0_config(1, AIO0CFG_MODE_INPUT, 0, AIO0CFG_MUX_VREF);
#elif (AIO0_MODE == AIO0_MODE_OUTPUT_VREF)
    afe_aio0_config(1, AIO0CFG_MODE_OUTPUT, 0, AIO0CFG_MUX_VREF);
#elif (AIO0_MODE == AIO0_MODE_OUTPUT_AFEMUX)
    // Configure AFEMUX = VCORE
    afe_afemux_config(0, 1);
    afe_afemux_select_mux(AFEMUXSEL_AFEMUX_VCORE);

    // Configure AIO
    afe_aio0_config(1, AIO0CFG_MODE_OUTPUT, 0, AIO0CFG_MUX_AFEMUXOUT);
#elif (AIO0_MODE == AIO0_MODE_OUTPUT_IMUX)
    // Configure IADC: DA gain x8
    afe_isense_diffamp_enable(1);
    afe_iadc_config(IADCCTL_DAGAIN_8, IADCCTL_IMUX_ISNSDA, 0);

    // Turn on DSG/CHG 
        // Configure PB0/PB1 as output to control DSG/CHG FETs
    PAC_GPIOB->OUT.P0 = 0;
    PAC_GPIOB->OUT.P1 = 0;
    PAC_GPIOB->MODE.P0 = 0x1; // push/pull output
    PAC_GPIOB->MODE.P1 = 0x1; // push/pull output
		PAC_GPIOB->OUT.P0 = 1;
		PAC_GPIOB->OUT.P1 = 1;
    // Configure AIO
    afe_aio0_config(1, AIO0CFG_MODE_OUTPUT, 0, AIO0CFG_MUX_IMUXOUT);
#elif (AIO0_MODE == AIO0_MODE_OUTPUT_VBMUX)
    // 10 cells configuration
    afe_cell_enable(0x3FF);
    
    // Select VBMUX
    afe_vadc_config(VADCCTL_VBMUX_VB1, 0);

    // Configure AIO
    afe_aio0_config(1, AIO0CFG_MODE_OUTPUT, 0, AIO0CFG_MUX_VBMUXOUT);
#endif

}

void aio0_test(void)
{
#if (AIO0_MODE == AIO0_MODE_INPUT)
    pac_adc_start();
    while (PAC_ADC->ADCCTL.ADBUSY);
    printf("AIO0 input: %d \n", PAC_ADC->ADCRES.VAL);
#endif
}


