/**
 * @file 	pac_iadc_conversion.c
 * @version v1.0.0
 * @date	14 October 2022
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"
#include "pac_iadc_conversion.h"
#include "pac_driver_gpio.h"

float iadc_gain;
float iadc_offset;

void pac_iadc_conversion_init(void)
{
    uint32_t tmp1;
    int32_t tmp2;

    // Read IADC calibration data from INFO ROM
    tmp1 = *((uint32_t *)IADC_GAIN_INFO_ADDR);
    iadc_gain = (float)tmp1 * IADC_GAIN_FACTOR / SHUNT_RESISTOR_OHMS;

    tmp2 = *((int32_t *)IADC_OFFSET_INFO_ADDR);
    iadc_offset = (float)tmp2 * IADC_OFFSET_FACTOR / SHUNT_RESISTOR_OHMS;

    // Configure PB0/PB1 as output to control DSG/CHG and PB3 as input to check IADC conversion status
    PAC_GPIOB->OUT.P0 = 0;
    PAC_GPIOB->OUT.P1 = 0;
    PAC_GPIOB->MODE.P0 = 0x1; // push/pull output
    PAC_GPIOB->MODE.P1 = 0x1; // push/pull output
    PAC_GPIOB->MODE.P3 = 0x0; // push/pull output
		PAC_SCC->PBPDEN.P3 = 0x1;
	// Configure SOC Bridge for talking to AFE
	pac_tile_socbridge_config(1, 0);

    // Configure AFE
    afe_set_mcu_alive(1);
    afe_signal_manager_enable(1);
    afe_set_driver_mode(DRVCTL_DRVMODE_ENABLED);
    afe_charge_pump_enable(1);

    // Configure IADC
    afe_current_adc_enable(1);
    afe_isense_diffamp_enable(1);
    afe_iadc_config(IADCCTL_DAGAIN, IADCCTL_IMUX_ISNSDA, 0);
}

void pac_iadc_conversion(void)
{
    uint32_t adc_result;
    float current;

    afe_iadc_restart_conversion(IADCCTL_DAGAIN, IADCCTL_IMUX_ISNSDA);
    while (PAC_GPIOB->IN.P3 == 1);

    adc_result = afe_iadc_get_result();
    current = adc_result * iadc_gain + iadc_offset;

    printf("Current (A): %.3f\n", current);
}


