/**
 * @file 	pac_vadc_conversion.c
 * @version v1.0.0
 * @date	14 October 2022
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"
#include "pac_vadc_conversion.h"
#include "pac_driver_gpio.h"

float vb1_gain;
float vb1_offset;

void pac_vadc_conversion_init(void)
{
    uint32_t tmp1;
    int32_t tmp2;

    // Read VB1 calibration data from INFO ROM
    tmp1 = *((uint32_t *)CAL_VB01_INFO_ADDR);
    vb1_gain = (float)(tmp1 >> 14) * ONE_DIV_2_POWER_28F;

    tmp2 = ((int)(tmp1 & 0x3FFF) << 18) >> 18;
    vb1_offset = -6.25f - ((float)tmp2 * ONE_DIV_2_POWER_15F);

    // Configure PB0/PB1 as output to control DSG/CHG FETs
    PAC_GPIOB->OUT.P0 = 0;
    PAC_GPIOB->OUT.P1 = 0;
    PAC_GPIOB->MODE.P0 = 0x1; // push/pull output
    PAC_GPIOB->MODE.P1 = 0x1; // push/pull output

	// Configure SOC Bridge for talking to AFE
	pac_tile_socbridge_config(1, 0);

    // Configure AFE
    afe_set_mcu_alive(1);
    afe_signal_manager_enable(1);
    afe_set_driver_mode(DRVCTL_DRVMODE_ENABLED);
    afe_charge_pump_enable(1);

    // 10 cells configuration
    afe_cell_enable(0x3FF);
    
    // Configure VADC to measure VB1 voltage
    afe_voltage_adc_enable(1);
    afe_vadc_config(VADCCTL_VBMUX_VB1, 0);
}

void pac_vadc_conversion(void)
{
    uint32_t adc_result;
    float vb1_voltage;

    afe_vadc_restart_conversion(VADCCTL_VBMUX_VB1);
    while (PAC_GPIOA->IN.P4 == 1);

    adc_result = afe_vadc_get_result();
    vb1_voltage = adc_result * vb1_gain + vb1_offset;
    
    printf("VB1 voltage (V): %.3f\n", vb1_voltage);
}


