/**
 * @file 	pac_scp.c
 * @version v1.0.0
 * @date	14 October 2022
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"
#include "pac_scp.h"
#include "pac_driver_gpio.h"

void pac_scp_init(void)
{
    uint32_t tmp;

    // Configure PB0/PB1 as output to control DSG/CHG FETs
    PAC_GPIOB->OUT.P0 = 0;
    PAC_GPIOB->OUT.P1 = 0;
    PAC_GPIOB->MODE.P0 = 0x1; // push/pull output
    PAC_GPIOB->MODE.P1 = 0x1; // push/pull output

	// Configure SOC Bridge for talking to AFE
	pac_tile_socbridge_config(1, 0);

    // Configure AFE
    afe_set_mcu_alive(1);
    afe_signal_manager_enable(1);
    afe_set_driver_mode(DRVCTL_DRVMODE_ENABLED);
    afe_charge_pump_enable(1);

    // Configure SCP protection
    // code = I*R*max_code/Vref
    // rounding +0.5f
    tmp = (int)(SCP_AMPERE * SHUNT_RESISTOR_OHMS * SCPDAC_MAX_COUNT / SCPDAC_VREF + 0.5f);
    afe_scp_set_dac(tmp);
    afe_scp_config(SCPCFG_TIMEBASE_1US, SCPCFG_BLANKSF_1);

    // Enable SCP DAC comparator
    afe_scp_daccomp_enable(1);
    // Disable CHG and DSG FET when SCP occurs
    afe_protection_enable(0, 0, 0, 0, 1, 1);
    // Clear all sigfaults
    afe_sigfault_clear_status(0xFF);
    // Enable SCP fault
    afe_sigfault_enable(0, 0, 0, 1, 0);
}

void pac_scp_monitor(void)
{
    int scpfault = SIGFAULT_SCPFLT_Msk & afe_sigfault_get_status();
    scpfault = scpfault && 1;
    printf("SCP Fault status: %d\n", scpfault);
}


