/**
 * @file 	pac_occ.c
 * @version v1.0.0
 * @date	14 October 2022
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"
#include "pac_occ.h"
#include "pac_driver_gpio.h"

void pac_occ_init(void)
{
    float tmp;

    // Configure PB0/PB1 as output to control DSG/CHG FETs
    PAC_GPIOB->OUT.P0 = 0;
    PAC_GPIOB->OUT.P1 = 0;
    PAC_GPIOB->MODE.P0 = 0x1; // push/pull output
    PAC_GPIOB->MODE.P1 = 0x1; // push/pull output

	// Configure SOC Bridge for talking to AFE
	pac_tile_socbridge_config(1, 0);

    // Configure AFE
    afe_set_mcu_alive(1);
    afe_signal_manager_enable(1);
    afe_set_driver_mode(DRVCTL_DRVMODE_ENABLED);
    afe_charge_pump_enable(1);

    // Configure Diffamp: gain = 8
    afe_isense_diffamp_enable(1);
    afe_iadc_config(IADCCTL_DAGAIN_8, IADCCTL_IMUX_ISNSDA, 0);

    // Configure OCC protection
    // code = (I*R*diffamp_gain + diffamp_offset) * max_code / Vref
    tmp = OCC_AMPERE * SHUNT_RESISTOR_OHMS * 8; // diffamp_gain = 8
    tmp = (tmp + DIFFAMP_OFFSET) * OCCDAC_MAX_COUNT / OCCDAC_VREF;
    afe_occ_set_dac((uint32_t)(tmp + 0.5f));    // rounding +0.5f

    afe_occ_config(OCCCFG_TIMEBASE_2048US, OCCCFG_BLANKSF_1);

    // Enable OCC DAC comparator
    afe_occ_daccomp_enable(1);
    // Disable CHG FET when OCC occurs
    afe_protection_enable(0, 0, 0, 1, 0, 0);
    // Clear all sigfaults
    afe_sigfault_clear_status(0xFF);
    // Enable OCC fault
    afe_sigfault_enable(0, 0, 1, 0, 0);
}

void pac_occ_monitor(void)
{
    int occfault = SIGFAULT_OCCFLT_Msk & afe_sigfault_get_status();
    occfault = occfault && 1;
    printf("OCC Fault status: %d\n", occfault);
}


