//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================
 
#include "app.h"

// #define SYSCLOCK_ROSC
#define SYSCLOCK_CLKREF_4MHZ
// #define SYSCLOCK_EXTCLK
// #define SYSCLOCK_XTAL

void system_init(void)
{
    uint8_t clksrc_in;

    // Set Flash Lock to allow write access to MEMCTL register for configuring clocks
    PAC_MEMCTL->FLASHLOCK = FLASH_LOCK_ALLOW_WRITE_MEMCTL;
    
    // Turn on Flache Cache
    PAC_MEMCTL->MEMCTL.CACHEDIS = 0;                        //Enable Flash CACHE

    // Always make sure MCLK is set to use ROSC CLK when making changes to FRCLK, SCLK, or HCLK
    PAC_MEMCTL->MEMCTL.MCLKSEL = MEMCTL_MCLK_ROSCCLK;       // MCLK = ROSCCLK
    
    // Select SCLK = FRCLK before making changes to FRCLK or PLLCLK
    PAC_SCC->CCSCTL.SCLKMUXSEL = CCSCTL_SCLK_FRCLK;

    // Select Clock input
    #if defined (SYSCLOCK_ROSC)
        clksrc_in = CCSCTL_CLKIN_ROSC;
    #elif defined (SYSCLOCK_CLKREF_4MHZ)
        clksrc_in = CCSCTL_CLKIN_CLKREF;
    #elif defined (SYSCLOCK_EXTCLK)
        clksrc_in = CCSCTL_CLKIN_XTALCLK;
    #elif defined (SYSCLOCK_XTAL)
        clksrc_in = CCSCTL_CLKIN_EXTCLK;
    #endif
    
    // Select 4 MHz CLKREF for Free Running Clock FRCLK
    PAC_SCC->CCSCTL.FRCLKMUXSEL = clksrc_in;

    #if defined (SYSCLOCK_ROSC)
        pac_sys_pll_config_enable(8, 150, 0);                   // PLLCLK = 300 MHz = (16/8 * 150) /1
    #elif defined (SYSCLOCK_CLKREF_4MHZ)
        pac_sys_pll_config_enable(4, 300, 0);                   // PLLCLK = 300 MHz = (4/4 * 300) /1
    #endif

    // Configure SCLK=PLLCLK=300 MHz, HCLK=150 MHz, PCLK=150 MHz, ACLK=300 MHz and WaitStates;  Use default PCLKDIV=1
    PAC_SCC->CCSCTL.HCLKDIV = CCSCTL_HCLKDIV_DIV2;          // HCLK = 150 MHz = SCLK/2; when SCLK = PLLCLK
    PAC_SCC->CCSCTL.ACLKDIV = CCSCTL_ACLKDIV_DIV1;          // ACLK = 300 MHz = SCLK/1; when SCLK = PLLCLK
    PAC_MEMCTL->MEMCTL.WSTATE = 5 + 1;                      // Flash = 150/25 = 6 clocks = 5 WS; So, need 5 + 1 Extra WS
    PAC_SCC->CCSCTL.SCLKMUXSEL = CCSCTL_SCLK_PLLCLK;        // SCLK = PLLCLK

    // Set MCLK for Flash write & erase in addition to read
    PAC_MEMCTL->MEMCTL.MCLKDIV = MEMCTL_MCLK_DIV5;          // MCLK will = HCLK/5 when MCLKSEL = MEMCTL_MCLK_HCLKDIV
    PAC_MEMCTL->MEMCTL.MCLKSEL = MEMCTL_MCLK_HCLKDIV;       // MCLK = HCLK/5 = 30 MHz; allows reading and writing of Flash

    PAC_MEMCTL->FLASHLOCK = 0;                              // Disallow write access to MEMCTL

    // Enable GPIO Input clock synchronization; should always be enabled, especially when using GPIO edge based interrupts
    PAC_GPIOA->CLKSYNC.w = 0xFF;
    PAC_GPIOB->CLKSYNC.w = 0xFF;
    PAC_GPIOC->CLKSYNC.w = 0xFF;
    PAC_GPIOD->CLKSYNC.w = 0xFF;
    PAC_GPIOE->CLKSYNC.w = 0xFF;
    PAC_GPIOF->CLKSYNC.w = 0xFF;
    PAC_GPIOG->CLKSYNC.w = 0xFF;
}
