//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"

void rtc_read_date(unsigned char* year, unsigned char* month, unsigned char* day, unsigned char* week)
{
    unsigned char y, m, d, w;

    y = PAC_RTC->DATE.YEAR;
    m = PAC_RTC->DATE.MONTH;
    d = PAC_RTC->DATE.DAY;
    w = PAC_RTC->DATE.DAYOFWEEK;

    y = ((y/16)*10) + (y%16);
    m = ((m/16)*10) + (m%16);
    d = ((d/16)*10) + (d%16);
    w = ((w/16)*10) + (w%16);

    *year = y;
    *month = m;
    *day = d;
    *week = w;
}

void rtc_read_time(unsigned char* hour, unsigned char* minute, unsigned char* second)
{
    unsigned char h, m, s;

    h = PAC_RTC->TIME.HOUR;
    m = PAC_RTC->TIME.MINUTE;
    s = PAC_RTC->TIME.SECOND;

    h = ((h/16)*10) + (h%16);
    m = ((m/16)*10) + (m%16);
    s = ((s/16)*10) + (s%16);

    *hour = h;
    *minute = m;
    *second = s;
}

void rtc_set_date(unsigned char year, unsigned char month, unsigned char day, unsigned char week)
{
    PAC_RTC->DATESET.YEAR = year;
    PAC_RTC->DATESET.MONTH = month;
    PAC_RTC->DATESET.DAY = day;
    PAC_RTC->DATESET.DAYOFWEEK = week;
}

void rtc_set_time(unsigned char hour, unsigned char minute, unsigned char second)
{
    PAC_RTC->TIMESET.HOUR = hour;
    PAC_RTC->TIMESET.MINUTE = minute;
    PAC_RTC->TIMESET.SECOND = second;
}

void rtc_set_alarm(unsigned char week, unsigned char day, unsigned char hour, unsigned char minute)
{
    PAC_RTC->ALARMSET.DAYOFWEEKALARM = week; 
    PAC_RTC->ALARMSET.DAYOFWEEKALARMEN = 1;
    PAC_RTC->ALARMSET.DAYALARM = day; 
    PAC_RTC->ALARMSET.DAYALARMEN = 1;
    PAC_RTC->ALARMSET.HOURALARM = hour; 
    PAC_RTC->ALARMSET.HOURALARMEN = 1;
    PAC_RTC->ALARMSET.MINUTEALARM = minute;
    PAC_RTC->ALARMSET.MINUTEALARMEN = 1;
}

void configure_rtc(void)
{
    PAC_RTC->CTL.CLKDIV = RTC_CLKDIV_4;             // input clock divider to get a 1MHz RTCclock

    rtc_set_date(0x23, 0x08, 0x02, 0x01);           // date include year, month, day, week
    rtc_set_time(0x10, 0x15, 0x55);                 // time include hour, minute, second
    rtc_set_alarm(0x01, 0x02, 0x10, 0x17);          // setting an alarm time by week, day, hour, minute

    PAC_RTC->CTL.EN = 1;                            //  enable RTC
    PAC_RTC->CTL.SETCAL = 1;

    PAC_RTC->CTL.IE = 1;                            //  enable interrupt
    NVIC_EnableIRQ(RTC_IRQn);
    NVIC_SetPriority(RTC_IRQn, 1);
}
