//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"

//=====================================
// select test function
//=====================================
#define TEST_TIMER_BASE
// #define TEST_TIMER_SYNCHRONIZATION
// #define TEST_TIMER_PWM_OUTPUT
// #define TEST_TIMER_CAPTURE
// #define TEST_TIMER_QEP

//===============================================================================================================================
// PAC55xx devices have four timers. Every timer has many functions.
// In this application, the basic Timer module functionality can be tested.
//===============================================================================================================================
void timer_app(void)
{
    // Disable global interrupts until initialization is complete
    __disable_irq();

    system_init();
    uart_init(UARTC, 115200);
    gpiox_debug_init();

#ifdef TEST_TIMER_BASE
    //-----------------------test base timer-----------------------------//
    configure_timer_b();                            // PE0-->base interrupt     PE1-->PWM failing edge interrupt
#endif

#ifdef TEST_TIMER_SYNCHRONIZATION
    //-----------------------test synchronization mode-----------------------------//
    configure_timer_synchronization();              // PC7-->PWMB7   PE0-->PWMC4    PF2-->PWMD2
#endif

#ifdef TEST_TIMER_PWM_OUTPUT
    //------------------------test pwm output-----------------------------//
    configure_timer_c_compare_mode();               // PE0 output-->PWMC4
#endif

#ifdef TEST_TIMER_CAPTURE
    //--------------------------test capture------------------------------//
    configure_timer_c_pwm_output();                 // PE0 output-->PWMC0
    configure_timer_d_capture_mode();               // PE1 -->Input to PWMD1
#endif

#ifdef TEST_TIMER_QEP
    //--------------------------test qep---------------------------------//
    configure_timer_d_compare_mode();               // OUTPUT: PE0-->QEPPHASEA    PE1-->QEPPHASEB    PC7-->QEPINDEX
    configure_timer_c_qep_mode();                   // INPUT:  PC5-->TCQEPPHA     PC6-->TCQEPPHB     PC4-->TCQEPIDX
#endif

    // Enable global interrupts
    __enable_irq();

    while (1)
    {
        delay_times(500000);

#ifdef TEST_TIMER_BASE
        //-----------------------test base timer-----------------------------//
        printf("\r\n------------- Timer Base/PWM interrupt test -------------\t\r\n\r\n");
        printf("TimerB frequency = %d Hz \t\r\n", TIMER_B_FREQUENCY);
#endif

#ifdef TEST_TIMER_SYNCHRONIZATION
        //-----------------------test synchronization mode-----------------------------//
        printf("------------- Timer Synchronization mode test ------------\t\r\n");
        printf("TimerB frequency = %d Hz \t\r\n", TIMER_SYNC_TEST_FREQ);
        printf("TimerC frequency = %d Hz \t\r\n", TIMER_SYNC_TEST_FREQ);
        printf("TimerD frequency = %d Hz \t\r\n\r\n", TIMER_SYNC_TEST_FREQ);
#endif

#ifdef TEST_TIMER_PWM_OUTPUT
        //------------------------test pwm output-----------------------------//
        printf("------------- PWM output test ------------\t\r\n\r\n");
        printf("TimerC frequency = %d Hz \t\r\n", PWM_TEST_FREQ);
        printf("PWMC4 duty = %d percent \t\r\n", 50);
#endif

#ifdef TEST_TIMER_CAPTURE
        //-----------------------test capture-----------------------------
        printf("------------- Timer Capture test -----------\t\r\n");
        peripheral.timer_a_ctr0_capture_1 = PAC_TIMERD->CCTR1.CAP1;
        printf("CAPTURE = %d        \t\r\n\r\n", peripheral.timer_a_ctr0_capture_1);
        // peripheral.timer_a_ctr0_capture_2 = PAC_TIMERD->CCTR1.CAP2;
        // printf("CAP1 = %d       CAP2 = %d \t\r\n\r\n", peripheral.timer_a_ctr0_capture_1, peripheral.timer_a_ctr0_capture_2);
#endif

#ifdef TEST_TIMER_QEP
        //-----------------------test qep-----------------------------
        printf("------------- Timer QEP test -----------\t\r\n\r\n");
        if (PAC_TIMERC->QEPCTR != peripheral.timer_c_qep_ticks)
        {
            peripheral.timer_c_qep_direction = PAC_TIMERC->QEPCTL.DIR;
            peripheral.timer_c_qep_ticks =  PAC_TIMERC->QEPCTR;
            printf("QEP_DIRECTION = %d     QEP_TICKS = %d \t\r\n\r\n", peripheral.timer_c_qep_direction, peripheral.timer_c_qep_ticks);
        }
#endif
    }
}
