//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "app.h"

void configure_timer_b(void)
{
    // Configure Timer B Controls
    pac_timer_clock_config(TimerB, TXCTL_CS_ACLK, TXCTL_PS_DIV1);       // Configure timer clock input for ACLK, /1 divider
    pac_timer_base_config(TimerB, TIMER_B_PERIOD_COUNTS, AUTO_RELOAD, TxCTL_MODE_UP, TIMER_SLAVE_SYNC_DISABLE);     // Configure timer frequency and count mode
    pac_timer_b_ccctr0_value_set(TIMER_B_PERIOD_COUNTS >>1);            // Set PWMB0 pwm duty cycle

    // Enable TBCCR0 interrupt at highest priority
    PAC_TIMERB->CCTL0.CCMODE = TXCCTL_CCMODE_COMPARE;
    PAC_TIMERB->CCTL0.CCINTEDGE = TXCCTL_CCINT_FALLING_EDGE; 
    PAC_TIMERB->CCTL0.CCINTEN = 1;                                      // Enable interrupts on TACCR0
    PAC_TIMERB->INT.CCR0IF = 1;                                         // Clear PWMB0 interrupt flag
    PAC_TIMERB->CTL.BASEIE = 1;                                         // Enable base timer
    PAC_TIMERB->INT.BASEIF = 1;                                         // Clear timer base interrupt flag
    NVIC_EnableIRQ(TimerB_IRQn);
    NVIC_SetPriority(TimerB_IRQn, 1);
}

void configure_timer_synchronization(void)
{
    // Configure Timer A Controls
    pac_timer_clock_config(TimerA, TXCTL_CS_ACLK, TXCTL_PS_DIV1);       // Configure timer clock input for ACLK, /1 divider
    pac_timer_base_config(TimerA, TIMER_SYNC_PERIOD_COUNTS, AUTO_RELOAD, TxCTL_MODE_DISABLED, TIMER_SLAVE_SYNC_DISABLE);   // Configure timer frequency and count mode

    pac_timer_clock_config(TimerB, TXCTL_CS_ACLK, TXCTL_PS_DIV1);       // Configure timer clock input for ACLK, /1 divider
    pac_timer_base_config(TimerB, TIMER_SYNC_PERIOD_COUNTS, AUTO_RELOAD, TxCTL_MODE_DISABLED, TIMER_SLAVE_SYNC_ENABLE);    // Configure timer frequency and count mode

    pac_timer_clock_config(TimerC, TXCTL_CS_ACLK, TXCTL_PS_DIV1);       // Configure timer clock input for ACLK, /1 divider
    pac_timer_base_config(TimerC, TIMER_SYNC_PERIOD_COUNTS, AUTO_RELOAD, TxCTL_MODE_DISABLED, TIMER_SLAVE_SYNC_ENABLE);    // Configure timer frequency and count mode

    pac_timer_clock_config(TimerD, TXCTL_CS_ACLK, TXCTL_PS_DIV1);       // Configure timer clock input for ACLK, /1 divider
    pac_timer_base_config(TimerD, TIMER_SYNC_PERIOD_COUNTS, AUTO_RELOAD, TxCTL_MODE_DISABLED, TIMER_SLAVE_SYNC_ENABLE);    // Configure timer frequency and count mode

    PAC_TIMERA->CTL.PRDLATCH = TXCTL_PRDLATCH_TXCTR_IM;
    PAC_TIMERB->CTL.PRDLATCH = TXCTL_PRDLATCH_TXCTR_IM;
    PAC_TIMERC->CTL.PRDLATCH = TXCTL_PRDLATCH_TXCTR_IM;
    PAC_TIMERD->CTL.PRDLATCH = TXCTL_PRDLATCH_TXCTR_IM;

    PAC_TIMERA->CTL.MODE = TXCTL_MODE_UPDOWN;
    PAC_TIMERB->CTL.MODE = TXCTL_MODE_UPDOWN;
    PAC_TIMERC->CTL.MODE = TXCTL_MODE_UPDOWN;
    PAC_TIMERD->CTL.MODE = TXCTL_MODE_UPDOWN;

    PAC_TIMERA->CTL.CLR = CLEAR_COUNT;
    PAC_TIMERA->CTL.CLR = NOT_CLEAR_COUNT;
    
//    PAC_TIMERA->CCTR0.CTR = (300000000/20000) >>1;                      //PWMA0 duty --> 50%
    PAC_TIMERB->CCTR4.CTR = TIMER_SYNC_PERIOD_COUNTS >>1;				//PWMB4 duty --> 50%
    PAC_TIMERC->CCTR4.CTR = TIMER_SYNC_PERIOD_COUNTS >>1;				//PWMC4 duty --> 50%
    PAC_TIMERD->CCTR4.CTR = TIMER_SYNC_PERIOD_COUNTS >>1;				//PWMD4 duty --> 50%

//    PAC_TIMERA->CTR.w = 0;
//    PAC_TIMERB->CTR.w = 500;
//    PAC_TIMERC->CTR.w = 1000;
//    PAC_TIMERD->CTR.w = 1500;

//    PAC_SCC->PBMUXSEL.P0 = 1;                                           // PB0-->PWMA0
//    PAC_GPIOB->MODE.P0 = 1;                                             // PB0-->Pull-Pull output
//    PAC_GPIOB->OUTMASK.P0 = 0;                                          // PB0-->enable output

    PAC_SCC->PDMUXSEL.P4 =  1;                                          // PD4-->PWMB4
    PAC_GPIOD->MODE.P4 = 1;                                             // PD4-->Pull-Pull output
    PAC_GPIOD->OUTMASK.P4 = 0;                                          // PD4-->enable output

    PAC_SCC->PEMUXSEL.P0 =  1;                                          // PE0-->PWMC4
    PAC_GPIOE->MODE.P0 = 1;                                             // PE0-->Pull-Pull output
    PAC_GPIOE->OUTMASK.P0 = 0;                                          // PE0-->enable output

    PAC_SCC->PEMUXSEL.P4 =  1;                                          // PE4-->PWMD4
    PAC_GPIOE->MODE.P4 = 1;                                             // PE4-->Pull-Pull output
    PAC_GPIOE->OUTMASK.P4 = 0;                                          // PE4-->enable output
}
