//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================
#include "app.h"

#ifndef ADC_AIO_FUNC_H
#define ADC_AIO_FUNC_H

#define IRQNUM_ADC0                     0                                           // Assert ADC0 IRQ to NVIC when the sequence has completed conversion
#define IRQNUM_ADC1                     1                                           // Assert ADC1 IRQ to NVIC when the sequence has completed conversion
#define IRQNUM_ADC2                     2                                           // Assert ADC2 IRQ to NVIC when the sequence has completed conversion
#define IRQNUM_ADC3                     3                                           // Assert ADC3 IRQ to NVIC when the sequence has completed conversion

#define EMUXC_NONE                      0                                           // Do not send EMUX Command
#define EMUXC_BSH                       1                                           // Send EMUX command before sample and hold
#define EMUXC_AEOC                      2                                           // Send EMUX command after conversion complete

#define DELAY_0                         0
#define DELAY_4                         1
#define DELAY_8                         2
#define DELAY_16                        3

#define IRQ_DIS                         0
#define IRQ_EN                          1

#define SEQDONE                         1

#ifdef CAFE_ARCH1
#define SEND_AFTER_CONVERSION   1   // Send EMUX Command after conversion

#define EMUX_VCORE          (0x40 | AFEMUXSEL_AFEMUX_VCORE)             // VCORE
#define EMUX_VCORE_DIV2P5   (0x40 | AFEMUXSEL_AFEMUX_VCORE_DIV2P5)      // VCORE*4/10
#define EMUX_VDDA_DIV2P5    (0x40 | AFEMUXSEL_AFEMUX_VDDA_DIV2P5)       // VDDA*4/10
#define EMUX_VCCIO_DIV2P5   (0x40 | AFEMUXSEL_AFEMUX_VCCIO_DIV2P5)      // VCCIO*4/10
#define EMUX_VSYS_DIV2P5    (0x40 | AFEMUXSEL_AFEMUX_VSYS_DIV2P5)       // VSYS*4/10
#define EMUX_ISENSE         (0x40 | AFEMUXSEL_AFEMUX_ISENSE)            // ISENSE
#define EMUX_VPTAT          (0x40 | AFEMUXSEL_AFEMUX_VPTAT)             // VPTAT
#define EMUX_VP_DIV10       (0x40 | AFEMUXSEL_AFEMUX_VP_DIV10)          // VP/10
#define EMUX_VREF_DIV2      (0x40 | AFEMUXSEL_AFEMUX_VREF_DIV2)         // VREF/2
#define EMUX_FUSE_DIV10     (0x40 | AFEMUXSEL_AFEMUX_FUSE_DIV10)        // FUSE/10
#define EMUX_CHG_DIV50      (0x40 | AFEMUXSEL_AFEMUX_CHG_DIV50)         // CHG/50
#define EMUX_DSG_DIV50      (0x40 | AFEMUXSEL_AFEMUX_DSG_DIV50)         // DSG/50
#define EMUX_BAT_DIV50      (0x40 | AFEMUXSEL_AFEMUX_BAT_DIV50)         // BAT/50
#define EMUX_AIO0A          (0x40 | AFEMUXSEL_AFEMUX_AIO0A)             // AIO0A
#define EMUX_LOADDET        (0x40 | AFEMUXSEL_AFEMUX_LOADDET)           // LOADDET
#define EMUX_SCPDAC         (0x40 | AFEMUXSEL_AFEMUX_SCPDAC)            // SCPDAC
#define EMUX_OCCDAC         (0x40 | AFEMUXSEL_AFEMUX_OCCDAC)            // OCCDAC
#define EMUX_OCDDAC         (0x40 | AFEMUXSEL_AFEMUX_OCDDAC)            // OCDDAC
#define EMUX_BATOVDAC_DIV2  (0x40 | AFEMUXSEL_AFEMUX_BATOVDAC_DIV2)     // BATOVDAC/2
#define EMUX_VIN_DIV50      (0x40 | AFEMUXSEL_AFEMUX_VIN_DIV50)         // VIN/50
#define EMUX_PACK_DIV50     (0x40 | AFEMUXSEL_AFEMUX_PACK_DIV50)        // PACK/50
#define EMUX_VCP_DIV50      (0x40 | AFEMUXSEL_AFEMUX_VCP_DIV50)         // VCP/50

#endif

extern volatile int16_t adc_val[24];
extern uint8_t fault_oc_flag;

void configurable_analog_front_end_init(void);

void test_manual_adc(void);
void adc_manual_init(void);
void adc_manual_convert_vcore(void);
void adc_manual_convert_vcc33(void);
void adc_manual_convert_vccio(void);
void adc_manual_convert_vsys(void);
void adc_manual_convert_vp(void);
void adc_manual_convert_vref(void);
void adc_manual_convert_temperature(void);
void adc_manual_sample(void);

void init_float_adc_gain_and_offset(void);
void init_fixed_point_adc_gain_and_offset(void);
void adc_manual_convert_vcore_calibrated(void);

void test_adc_dtse_sequence_example_1(void);
void init_adc_dtse_sequence_example_1(void);
void config_adc_dtse_sequence_example_1(void);

void test_adc_dtse_multi_sequence(void);
void init_adc_dtse_multi_sequence(void);
void config_adc_dtse_multi_sequence(void);


#endif
