//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#ifndef APP_H
#define APP_H

#include "config.h"     // config.h includes a device header file (example pac25140.h), which is needed for Tile headers below
#include "common.h"

#include "stdio.h"
#include "pac.h"

#include "pac_adc.h"
#include "pac_can.h"
#include "pac_crc.h"
#include "pac_gp_timers.h"
#include "pac_gpio.h"
#include "pac_i2c.h"
#include "pac_memory.h"
#include "pac_rtc.h"
#include "pac_scc.h"
#include "pac_ssp.h"
#include "pac_timers.h"
#include "pac_uart.h"
#include "pac_wwdt.h"

#include "pac_driver_adc.h"
#include "pac_driver_config.h"
#include "pac_driver_gpio.h"
#include "pac_driver_memory.h"
#include "pac_driver_socbridge.h"
#include "pac_driver_system.h"
#include "pac_driver_tile.h"
#include "pac_driver_timer.h"
#include "pac_driver_uart.h"

#include "adc_aio_func.h"
#include "can_func.h"
#include "crc_func.h"
#include "flash_func.h"
#include "gpio_func.h"
#include "gptimera_func.h"
#include "gptimerb_func.h"
#include "i2c_func.h"
#include "rtc_func.h"
#include "ssp_func.h"
#include "sysclk_func.h"
#include "timer_func.h"
#include "uart_func.h"
#include "wwdt_func.h"
#include "sleep.h"
#include "user.h"
#include "code_protect_func.h"

#include "peripheral_variables.h"

#include "pac_afemux.h"
#include "pac_driver_control.h"
#include "pac_hibernate.h"
#include "pac_aio0.h"
#include "pac_iadc_conversion.h"
#include "pac_vadc_conversion.h"
#include "pac_scp.h"
#include "pac_occ.h"
#include "pac_ocd.h"
#include "pac_batov.h"
#include "pac_power_fault.h"
#include "pac_temp_fault.h"
#include "pac_cell_balancing.h"
#include "pac_wwdt.h"

void adc_app(void);
//void aio_app(void);
void can_app(void);
void crc_app(void);
void flash_app(void);
void gpio_app(void);
void gptimera_app(void);
void gptimerb_app(void);
void hib_app(void);
void i2c_app(void);
void power_app(void);
void rtc_app(void);
void ssp_app(void);
void sysclk_app(void);
void timer_app(void);
void uart_app(void);
void wwdt_app(void);
void sleep_app(void);
void code_protect_app(void);
void timer_test(void);
void pwm_test(void);
void ramfunc_app(void);

void pac_afemux_test(void);
void pac_driver_control_test(void);
void pac_hibernate_test(void);
void pac_aio0_test(void);
void pac_iadc_conversion_test(void);
void pac_vadc_conversion_test(void);
void pac_scp_test(void);
void pac_occ_test(void);
void pac_ocd_test(void);
void pac_batov_test(void);
void pac_power_fault_test(void);
void pac_temp_fault_test(void);
void pac_cell_balancing_test(void);
void pac_clkout_test(void);
void pac_afe_wwdt_test(void);
void socbus_bridge_test(void);

#endif
