/**
 * @file bms_driver.c
 * @brief 
 */

//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "bms_driver.h"

//=============================================================================
//
// Primary Control and Status
//
//=============================================================================

void afe_signal_manager_enable(uint32_t enabled)
{
    pac_tile_register_write_bits(ADDR_AFECTL1, AFECTL1_SIGEN_Pos, AFECTL1_SIGEN_Msk, enabled);
}

void afe_charge_pump_enable(uint32_t enabled)
{
    pac_tile_register_write_bits(ADDR_AFECTL1, AFECTL1_HVCPEN_Pos, AFECTL1_HVCPEN_Msk, enabled);
}

void afe_load_detect_enable(uint32_t enabled)
{
    pac_tile_register_write_bits(ADDR_AFECTL1, AFECTL1_LOADDETEN_Pos, AFECTL1_LOADDETEN_Msk,enabled);
}

void afe_scpdac2vadc(uint32_t enable)
{
    pac_tile_register_write_bits(ADDR_AFECTL1, AFECTL1_SCPDAC2VADC_Pos, AFECTL1_SCPDAC2VADC_Msk, enable);
}

void afe_dis_cpok_qual(uint32_t val)
{
    pac_tile_register_write_bits(ADDR_AFECTL1, AFECTL1_Dis_CPOK_QUAL_Pos, AFECTL1_Dis_CPOK_QUAL_Msk, val);
}

void afe_set_mcu_alive(uint32_t alive)
{
    pac_tile_register_write_bits(ADDR_AFECTL1, AFECTL1_MCUALIVE_Pos, AFECTL1_MCUALIVE_Msk, alive);
}

void afe_assert_nrst()
{
    pac_tile_register_write_bits(ADDR_AFECTL1, AFECTL1_SRST_Pos, AFECTL1_SRST_Msk, AFE_RESET);
}

void afe_hvbuck_set_switching_frequency(AFECTL2_HVBUCK_FREQSEL_Type frequency)
{
    pac_tile_register_write_bits(ADDR_AFECTL2, AFECTL2_BK_FREQ_Pos, AFECTL2_BK_FREQ_Msk, frequency);
}

void afe_set_driver_mode(DRVCTL_DRVMODE_Type mode)
{
    pac_tile_register_write(ADDR_DRVCTL, mode);
}

void afe_afemux_config(uint32_t emux_en, uint32_t buff_en)
{
    uint32_t data;
    data = (buff_en << 1) | emux_en;
    pac_tile_register_write(ADDR_AFEMUXCTL, (uint8_t)data);
}

void afe_afemux_select_mux(AFEMUXSEL_AFEMUX_Type mux_type)
{
    pac_tile_register_write(ADDR_AFEMUXSEL, mux_type);
}

void afe_hibernate_config(uint32_t pb_wake_en, uint32_t pack_wake_en, HIBCTL_PACKWAKEVREF_VOLSEL_Type pack_wake_vref, HIBCTL_WAKESRC_Type wake_src, HIBCTL_WAKETIMER_Type wake_timer)
{
    uint32_t data;
    data = (pb_wake_en << HIBCTL_PBWAKEEN_Pos) | (pack_wake_en << HIBCTL_PACKWAKEEN_Pos) | (pack_wake_vref << HIBCTL_PACKWAKEVREF_Pos) | (wake_src << HIBCTL_WAKESRC_Pos) | (wake_timer << HIBCTL_WUTIMER_Pos);
    pac_tile_register_write(ADDR_HIBCTL, (uint8_t)data);
}

void afe_hibernate_enter()
{
    pac_tile_register_write(ADDR_HIBENTER, (1 << HIBENTER_HIB_Pos));
}

uint32_t afe_reset_get_status()
{
    uint32_t data;
    data = pac_tile_register_read(ADDR_RSTSTAT) & 0x1F;
    return data;
}

void afe_reset_clear_status(uint32_t status_mask)
{
    pac_tile_register_write(ADDR_RSTSTAT, (uint8_t)status_mask);
}

void afe_reset_clear_fltrst()
{
    pac_tile_register_write(ADDR_RSTSTAT, RSTSTAT_FLTRST_Msk);
}

void afe_reset_clear_softrst()
{
    pac_tile_register_write(ADDR_RSTSTAT, RSTSTAT_SOFTRST_Msk);
}

void afe_reset_clear_wdtrst()
{
    pac_tile_register_write(ADDR_RSTSTAT, RSTSTAT_WDTRST_Msk);
}

void afe_reset_clear_hibrst()
{
    pac_tile_register_write(ADDR_RSTSTAT, RSTSTAT_HIBRST_Msk);
}

void afe_reset_clear_pbrst()
{
    pac_tile_register_write(ADDR_RSTSTAT, RSTSTAT_PBRST_Msk);    
}

void afe_pushbtn_config(uint32_t enabled, uint32_t reset_en, PB_DEGLITCH_TIMESEL_Type deglitch_time, PB_POLARITY_Type polarity, uint32_t pb_int_en)
{
    uint32_t data;
    data = (enabled << PB_PBEN_Pos) | (reset_en << PB_PBRSTEN_Pos) | (deglitch_time << PB_PBDT_Pos) | (polarity << PB_PBPOL_Pos) | (pb_int_en << PB_PBINTEN_Pos);
    pac_tile_register_write(ADDR_PB, (uint8_t)data);
}

void afe_pushbtn_enable(uint32_t enabled)
{
    pac_tile_register_write_bits(ADDR_PB, PB_PBEN_Pos, PB_PBEN_Msk, enabled);
}

uint32_t afe_pushbtn_get_int_flag()
{    
    uint32_t data;
    data = pac_tile_register_read(ADDR_PB) & PB_PBINTF_Msk;
    return data != 0;
}

void afe_pushbtn_clear_int_flag()
{
    pac_tile_register_write_bits(ADDR_PB, PB_PBINTF_Pos, PB_PBINTF_Msk, AFE_W1C);
}

uint32_t afe_pushbtn_get_status()
{
    uint32_t data;
    data = pac_tile_register_read(ADDR_PB) & PB_PBSTAT_RTS_Msk;
    return data != 0;
}

void afe_aio0_config(uint32_t enabled, AIO0CFG_MODE_Type mode, uint32_t swap_offset, AIO0CFG_MUX_Type mux_type)
{
    uint32_t data;
    data = (enabled << AIO0CFG_AIO0EN_Pos) | (mode << AIO0CFG_MODE_Pos) | (swap_offset << AIO0CFG_SWAP_Pos) | (mux_type << AIO0CFG_MUXSEL_Pos);
    pac_tile_register_write(ADDR_AIO0CFG, (uint8_t)data);
}

void afe_aio0_swap_offset(uint32_t swap_offset)
{
    pac_tile_register_write_bits(ADDR_AIO0CFG, AIO0CFG_SWAP_Pos, AIO0CFG_SWAP_Msk, swap_offset);
}

void afe_aio0_enable(uint32_t enabled)
{
    pac_tile_register_write_bits(ADDR_AIO0CFG, AIO0CFG_AIO0EN_Pos, AIO0CFG_AIO0EN_Msk, enabled);
}

void afe_batov_daccomp_enable(uint32_t enabled)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write_bits(ADDR_SIGMGRCTL1, SIGMGRCTL1_BATOVEN_Pos, SIGMGRCTL1_BATOVEN_Msk, enabled);
}

void afe_ocd_daccomp_enable(uint32_t enabled)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write_bits(ADDR_SIGMGRCTL1, SIGMGRCTL1_OCDEN_Pos, SIGMGRCTL1_OCDEN_Msk, enabled);
}

void afe_occ_daccomp_enable(uint32_t enabled)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write_bits(ADDR_SIGMGRCTL1, SIGMGRCTL1_OCCEN_Pos, SIGMGRCTL1_OCCEN_Msk, enabled);
}

void afe_scp_daccomp_enable(uint32_t enabled)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write_bits(ADDR_SIGMGRCTL1, SIGMGRCTL1_SCPEN_Pos, SIGMGRCTL1_SCPEN_Msk, enabled);
}

void afe_isense_diffamp_enable(uint32_t enabled)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write_bits(ADDR_SIGMGRCTL1, SIGMGRCTL1_DAEN_Pos, SIGMGRCTL1_DAEN_Msk, enabled);
}

void afe_voltage_adc_enable(uint32_t enabled)
{
    pac_tile_register_write_bits(ADDR_SIGMGRCTL2, SIGMGRCTL2_VADCEN_Pos, SIGMGRCTL2_VADCEN_Msk, enabled);
}

void afe_current_adc_enable(uint32_t enabled)
{   
    pac_tile_register_write_bits(ADDR_SIGMGRCTL2, SIGMGRCTL2_IADCEN_Pos, SIGMGRCTL2_IADCEN_Msk, enabled);
}

void afe_pushbtn_passthrough_enable(uint32_t enabled)
{
    pac_tile_register_write_bits(ADDR_SIGMGRCTL2, SIGMGRCTL2_PBPTEN_Pos, SIGMGRCTL2_PBPTEN_Msk, enabled);
}

void afe_protection_enable(uint32_t batovd_prot_en, uint32_t batovc_prot_en, uint32_t ocd_prot_en, uint32_t occ_prot_en, uint32_t scpd_prot_en, uint32_t scpc_prot_en)
{
    uint32_t data;
    data = (batovd_prot_en << PROTEN_BATOVDPROTEN_Pos) | (batovc_prot_en << PROTEN_BATOVCPROTEN_Pos) | (ocd_prot_en << PROTEN_OCDPROTEN_Pos) | (occ_prot_en << PROTEN_OCCPROTEN_Pos) | (scpd_prot_en << PROTEN_SCPDPROTEN_Pos) | (scpc_prot_en << PROTEN_SCPCPROTEN_Pos);
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_PROTEN, (uint8_t)data);
}

void afe_fuse_enable(uint32_t enabled)
{
    uint32_t data;
    data = (FUSE_KEY << 1) | enabled; 
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_FUSE, (uint8_t)data);
}

void afe_pwrfault_enable_int(uint32_t hvcp_fault_en, uint32_t vp_fault_en, uint32_t vsys_fault_en, uint32_t vcore_fault_en, uint32_t vdda_fault_en, uint32_t vccio_fault_en, uint32_t drv_fault_en)
{
    uint32_t data;
    data = (hvcp_fault_en << PWRFAULTEN_HVCPFLTEN_Pos) | (vp_fault_en << PWRFAULTEN_VPFLTEN_Pos) | (vsys_fault_en << PWRFAULTEN_VSYSFLTEN_Pos) | (vcore_fault_en << PWRFAULTEN_VCOREFLTEN_Pos) | (vdda_fault_en << PWRFAULTEN_VDDAFLTEN_Pos) | (vccio_fault_en << PWRFAULTEN_VCCIOFLTEN_Pos) | (drv_fault_en << PWRFAULTEN_DRVFLTEN_Pos);
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_PWRFAULTEN, (uint8_t)data);
}

uint32_t afe_pwrfault_get_status()
{
    uint32_t data;
    data = pac_tile_register_read(ADDR_PWRFAULT) & 0x7F;
    return data;
}

void afe_pwrfault_clear_status(uint32_t fault_mask)
{
    pac_tile_register_write(ADDR_PWRFAULT, (uint8_t)fault_mask);
}

void afe_tempfault_enable(uint32_t twarn1_fault_en, uint32_t twarn2_fault_en, uint32_t tmp_fault_en, uint32_t twarn1_cbd_en, uint32_t twarn2_cbd_en)
{
    uint32_t data;
    data = (twarn1_fault_en << TEMPFAULTEN_TWARN1EN_Pos) | 
        (twarn2_fault_en << TEMPFAULTEN_TWARN2EN_Pos) | 
        (tmp_fault_en << TEMPFAULTEN_TMPFLTEN_Pos) | 
        (twarn1_cbd_en << TEMPFAULTEN_TWARN1CBDEN_Pos) | 
        (twarn2_cbd_en << TEMPFAULTEN_TWARN2CBDEN_Pos);
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_TEMPFAULTEN, (uint8_t)data);
}

uint32_t afe_tempfault_get_status()
{
    uint32_t data;
    data = pac_tile_register_read(ADDR_TEMPFAULT) & 0xC7;
    return data;
}

void afe_tempfault_clear_status(uint32_t fault_mask)
{
    pac_tile_register_write(ADDR_TEMPFAULT, (uint8_t)fault_mask);
}

void afe_sigfault_enable(uint32_t batov_fault_en, uint32_t ocd_fault_en, uint32_t occ_fault_en, uint32_t scp_fault_en, uint32_t emux_fault_en)
{
    uint32_t value = (batov_fault_en << SIGFAULTEN_BATOVFLTEN_Pos)      |
                    (ocd_fault_en << SIGFAULTEN_OCDFLTEN_Pos)           |
                    (occ_fault_en << SIGFAULTEN_OCCFLTEN_Pos)           |
                    (scp_fault_en << SIGFAULTEN_SCPFLTEN_Pos)           |
                    (emux_fault_en << SIGFAULTEN_EMUXFLTEN_Pos);

    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_SIGFAULTEN, (uint8_t)value);
}

uint32_t afe_sigfault_get_status()
{
    return pac_tile_register_read(ADDR_SIGFAULT);
}

void afe_sigfault_clear_status(uint32_t fault_mask)
{
    pac_tile_register_write(ADDR_SIGFAULT, (uint8_t)fault_mask);
}

uint32_t afe_batrts_get_status()
{
    return pac_tile_register_read(ADDR_BATRTS);
}

//=============================================================================
//
// Battery Monitoring and Protection
//
//=============================================================================
void afe_batov_config(BATOVCFG_TIMEBASE_Type time_base, BATOVCFG_BLANKSF_Type scale_factor)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_BATOVCFG, (time_base << BATOVCFG_TIMEBASE_Pos) | (scale_factor << BATOVCFG_BLANKSF_Pos));
}

void afe_batov_set_dac(uint32_t value)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_BATOVDAC, (uint8_t)value);
}

void afe_vadc_config(VADCCTL_VBMUX_Type mux_type, uint32_t start)
{
    uint32_t value = (mux_type << VADCCTL_VBMUXSEL_Pos) | (start << VADCCTL_VADCSTART_Pos);
    pac_tile_register_write(ADDR_VADCCTL, (uint8_t)value);
}

void afe_vadc_start_conversion()
{
    pac_tile_register_write_bits(ADDR_VADCCTL, VADCCTL_VADCSTART_Pos, VADCCTL_VADCSTART_Msk, VADC_START);
}

uint32_t afe_vadc_get_busy_flag()
{
    uint32_t value = pac_tile_register_read(ADDR_VADCCTL);
    return (value & VADCCTL_VADCBUSY_Msk) != 0;
}

uint32_t afe_vadc_get_result()
{
    // while(afe_vadc_get_busy_flag());

    uint32_t res_lo = pac_tile_register_read(ADDR_VADCRESLO);
    uint32_t res_hi = pac_tile_register_read(ADDR_VADCRESHI);
    return res_lo | (res_hi << 8);
}

void afe_iadc_config(IADCCTL_DAGAIN_Type da_gain, IADCCTL_IMUX_Type mux_type, uint32_t start)
{
    uint32_t value = (da_gain << IADCCTL_DAGAIN_Pos)    | 
                    (mux_type << IADCCTL_IMUXSEL_Pos)   | 
                    (start << IADCCTL_IADCSTART_Pos);
    pac_tile_register_write(ADDR_IADCCTL, (uint8_t)value);
}

void afe_iadc_start_conversion()
{
    uint8_t value = pac_tile_register_read(ADDR_IADCCTL);
    value |= 0x80;
    pac_tile_register_write_bits(ADDR_IADCCTL, IADCCTL_IADCSTART_Pos, IADCCTL_IADCSTART_Msk, IADC_START);
}

uint32_t afe_iadc_get_busy_flag()
{
    uint32_t value = pac_tile_register_read(ADDR_IADCCTL);
    return (value & IADCCTL_IADCBUSY_Msk) != 0;
}

uint32_t afe_iadc_get_result()
{
    // while(afe_iadc_get_busy_flag());

    uint32_t res_lo = pac_tile_register_read(ADDR_IADCRESLO);
    uint32_t res_hi = pac_tile_register_read(ADDR_IADCRESHI);
    return res_lo | (res_hi << 8);
}

//=============================================================================
//
// Short Circuit Protection
//
//=============================================================================
void afe_scp_set_dac(uint32_t value)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_SCPDAC, (uint8_t)value);
}

void afe_scp_config(SCPCFG_TIMEBASE_Type time_base, SCPCFG_BLANKSF_Type scale_factor)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_SCPCFG, (time_base << SCPCFG_TIMEBASE_Pos) | (scale_factor << SCPCFG_BLANKSF_Pos));
}

//=============================================================================
//
// Over Current Charge Protection
//
//=========================================-====================================
void afe_occ_set_dac(uint32_t value)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_OCCDAC, (uint8_t)value);
}

void afe_occ_config(OCCCFG_TIMEBASE_Type time_base, OCCCFG_BLANKSF_Type scale_factor)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_OCCCFG, (time_base << OCCCFG_TIMEBASE_Pos) | (scale_factor << OCCCFG_BLANKSF_Pos));
}

//=============================================================================
//
// Over Current Discharge Protection
//
//=========================================-====================================
void afe_ocd_set_dac(uint32_t value)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_OCDDAC, (uint8_t)value);
}

void afe_ocd_config(OCDCFG_TIMEBASE_Type time_base, OCDCFG_BLANKSF_Type scale_factor)
{
    pac_tile_register_write(ADDR_PROT_KEY, PROT_KEY);
    pac_tile_register_write(ADDR_OCDCFG, (time_base << OCDCFG_TIMEBASE_Pos) | (scale_factor << OCDCFG_BLANKSF_Pos));
}

//=============================================================================
//
// Cell Enablement / Monitoring
//
//=========================================-====================================
void afe_cell_enable(uint32_t cell_bits)
{
    pac_tile_register_write(ADDR_CELLEN1, cell_bits & 0xFF);
    pac_tile_register_write(ADDR_CELLEN2, (cell_bits >> 8) & 0xFF);
    pac_tile_register_write(ADDR_CELLEN3, (cell_bits >> 16) & 0x0F);
}

void afe_cellbalance_enable_fet(uint32_t cell_bits)
{
    pac_tile_register_write(ADDR_CFGCB1, cell_bits & 0xFF);
    pac_tile_register_write(ADDR_CFGCB2, (cell_bits >> 8) & 0xFF);
    pac_tile_register_write(ADDR_CFGCB3, (cell_bits >> 16) & 0x0F);
}

//=============================================================================
//
// Miscellaneous
//
//=========================================-====================================
void afe_gpreg_set(uint32_t value)
{
    pac_tile_register_write(ADDR_GP, (uint8_t)value);
}

uint32_t afe_gpreg_get()
{
    return pac_tile_register_read(ADDR_GP);
}

void afe_clkout_config(uint32_t enabled, CLKOUTCFG_FREQSEL_Type freq_select)
{
    uint32_t value = (enabled << CLKOUTCFG_CLKOUTEN_Pos) | (freq_select << CLKOUTCFG_CLKOUTFREQ_Pos);
    pac_tile_register_write(ADDR_CLKOUTCFG, (uint8_t)value);
}

void afe_clkout_enable(uint32_t enabled)
{
    pac_tile_register_write_bits(ADDR_CLKOUTCFG, CLKOUTCFG_CLKOUTEN_Pos, CLKOUTCFG_CLKOUTEN_Msk, enabled);
}

void afe_watchdog_config(uint32_t enabled, WWDTCTL_CLKDIV_Type clk_div, uint32_t cdv, uint32_t window)
{
    uint32_t ctl = ((!WWDT_ENABLE) << WWDTCTL_EN_Pos)                   | 
                    (clk_div << WWDTCTL_CLKDIV_Pos)                     | 
                    (WWDT_ALLOW_WRITE_REGS_KEY << WWDTCTL_KEY_Pos);
    pac_tile_register_write(ADDR_WWDTCTL, ctl);

    pac_tile_register_write(ADDR_WWDTCDV, cdv);
    pac_tile_register_write(ADDR_WWDTWIN, window);

    ctl |= (enabled << WWDTCTL_EN_Pos);
    pac_tile_register_write(ADDR_WWDTCTL, (uint8_t)ctl);
}

void afe_watchdog_enable(uint32_t enabled)
{
    uint32_t value = pac_tile_register_read(ADDR_WWDTCTL);
    value &= ~(WWDTCTL_EN_Msk | WWDTCTL_KEY_Msk);
    value |= (enabled << WWDTCTL_EN_Pos) | (WWDT_ALLOW_WRITE_REGS_KEY << WWDTCTL_KEY_Pos);
    pac_tile_register_write(ADDR_WWDTCTL, (uint8_t)value);
}

uint32_t afe_watchdog_get_ctr()
{
    return pac_tile_register_read(ADDR_WWDTCTR);
}

void afe_watchdog_reset()
{
    pac_tile_register_write(ADDR_WWDTRST, WWDT_RESET_KEY);
}

void afe_iadc_restart_conversion(IADCCTL_DAGAIN_Type da_gain, IADCCTL_IMUX_Type mux_type)
{
    afe_iadc_config(da_gain, IADCCTL_IMUX_ISNSDA, !IADC_START);
    afe_iadc_config(da_gain, IADCCTL_IMUX_ISNSDA, IADC_START);
}

void afe_vadc_restart_conversion(VADCCTL_VBMUX_Type cell_index)
{
    afe_vadc_config((VADCCTL_VBMUX_Type)cell_index, !VADC_START);
    afe_vadc_config((VADCCTL_VBMUX_Type)cell_index, VADC_START);
}
