//=============================================================================
// Copyright (C) 2023, Qorvo, Inc.
//
// THIS SOFTWARE IS SUBJECT TO A SOURCE CODE LICENSE AGREEMENT WHICH PROVIDES,
// AMONG OTHER THINGS:  (i) THAT IT CAN BE USED ONLY TO ADAPT THE LICENSEE'S
// APPLICATION TO PAC PROCESSORS SUPPLIED BY QORVO, INC.;
// (ii) THAT  IT IS PROVIDED "AS IS" WITHOUT WARRANTY;  (iii) THAT
// QORVO, INC. IS NOT LIABLE FOR ANY INDIRECT DAMAGES OR FOR DIRECT
// DAMAGES EXCEEDING US$1,500;  AND (iv) THAT IT CAN BE DISCLOSED TO AND USED
// ONLY BY CERTAIN AUTHORIZED PERSONS.
//
//=============================================================================

#include "pac.h"
#include "pac_UART.h"
#include "pac_driver_uart.h"
#include <stdio.h>

#ifdef __GNUC__
    #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
    #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif

PUTCHAR_PROTOTYPE
{
#ifndef UART_TEST
    PAC_UARTC->THR.THR = ch;                // Transmit a char;
//    while (PAC_UARTC->IIR.INTID == 0)       // set whilst TX in progress
//        continue;
    while (PAC_UARTC->LSR.TEMT == 0);
#else
    PAC_UARTD->THR.THR = ch;                // Transmit a char;
    while (PAC_UARTD->LSR.TEMT == 0);
#endif
    return (ch);
}

#if defined ( __ICCARM__ )
#include <LowLevelIOInterface.h>

#pragma module_name = "?__write"

int MyLowLevelPutchar(int ch)
{
#ifndef UART_TEST
    PAC_UARTC->THR.THR = ch;                // Transmit a char;
//    while (PAC_UARTC->IIR.INTID == 0)       // set whilst TX in progress
//        continue;
    while (PAC_UARTC->LSR.TEMT == 0);
#else
    PAC_UARTD->THR.THR = ch;                // Transmit a char;
    while (PAC_UARTD->LSR.TEMT == 0);
#endif
    return (ch);
}

/*
 * If the __write implementation uses internal buffering, uncomment
 * the following line to ensure that we are called with "buffer" as 0
 * (i.e. flush) when the application terminates.
 */

size_t __write(int handle, const unsigned char * buffer, size_t size)
{
  /* Remove the #if #endif pair to enable the implementation */
#if 1

  size_t nChars = 0;

  if (buffer == 0)
  {
    /*
     * This means that we should flush internal buffers.  Since we
     * don't we just return.  (Remember, "handle" == -1 means that all
     * handles should be flushed.)
     */
    return 0;
  }

  /* This template only writes to "standard out" and "standard err",
   * for all other file handles it returns failure. */
  if (handle != _LLIO_STDOUT && handle != _LLIO_STDERR)
  {
    return _LLIO_ERROR;
  }

  for (/* Empty */; size != 0; --size)
  {
    if (MyLowLevelPutchar(*buffer++) < 0)
    {
      return _LLIO_ERROR;
    }

    ++nChars;
  }

  return nChars;

#else

  /* Always return error code when implementation is disabled. */
  return _LLIO_ERROR;

#endif

}

#endif
